from interface import Interface, implements
import sys, os
from ams_instrument import ams_instrument
import logging
import nidcpower
from ams_smu import ams_SMU_instrument

my_logger = logging.getLogger(__name__)


class ams_PXI_SMU(implements(ams_SMU_instrument)):
    """
    This class implements the SMU functionality for PXI SMU device
    """
    def __init__(self, device_dict: dict) -> object:
        """
        This method initializes the SMU based on the HW configuration file
        Device dict example:
        {
            "Instrument_name": "PXI_SMU_6V_500mA_slot02",
            "Type": "PXIe-4143",
            "Details": {
                "Voltage_max": 6,
                "Current_max": 500,
                "Reset": 1,
                "Channels": [
                    "0",
                    "1"
                ] 
            }
        }
        Args:
            device_dict (dict): dictionary of instrument settings

        Returns:
            object: ams_SMU_instrument object
        """
        self.ni_device_dict = device_dict
        ni_max_alias = device_dict['Instrument_name']
        ni_channels = device_dict['Details']['Channels']
        ni_reset = device_dict['Details']['Reset'] == 1
        self.smu_handle = nidcpower.Session(ni_max_alias, channels = ni_channels, reset = ni_reset)
        self.smu_handle.source_mode = nidcpower.SourceMode.SINGLE_POINT
        self.smu_handle.measure_when = nidcpower.MeasureWhen.ON_DEMAND
        for channel in self.ni_device_dict['Details']['Channels']:
            self.smu_handle.channels[channel].output_function = nidcpower.OutputFunction.DC_VOLTAGE
            self.smu_handle.channels[channel].current_limit = 0.1
            self.smu_handle.channels[channel].current_level_autorange = True
            self.smu_handle.channels[channel].voltage_level_autorange = True
            self.smu_handle.channels[channel].source_delay = 0
            self.smu_handle.channels[channel].voltage_level = 0
            self.smu_handle.channels[channel].output_enabled = False
        self.smu_handle.initiate()


    def source_voltage(self, voltage_level: float, current_limit: float, channel_name: str):
        """source_voltage 
        This method sources out voltage to a certain channel with a defined current limit

        Args:
            voltage_level (float): SMU terminal voltage
            current_limit (float): SMU current limit
            channel_name (str): Name of the channel on SMU in use
        """
        self.smu_handle.abort()
        self.smu_handle.channels[channel_name].output_function = nidcpower.OutputFunction.DC_VOLTAGE
        self.smu_handle.initiate()
        self.smu_handle.channels[channel_name].voltage_level = voltage_level
        self.smu_handle.channels[channel_name].current_limit = current_limit


    def source_current(self, current_level: float, voltage_limit: float, channel_name: str):
        """source_current 
        This method sources out current to a certain channel with a defined voltage limit

        Args:
            current_level (float): SMU terminal current
            voltage_limit (float): SMU voltage limit
            channel_name (str): Name of the channel on SMU in use
        """
        self.smu_handle.abort()
        self.smu_handle.channels[channel_name].output_function = nidcpower.OutputFunction.DC_CURRENT
        self.smu_handle.initiate()
        self.smu_handle.channels[channel_name].current_level = current_level
        self.smu_handle.channels[channel_name].voltage_limit = voltage_limit


    def measure_voltage(
        self, 
        channel_name: str,
        range: str=None, 
        duration: float=None, 
        sampling_rate: float=None, 
        number_of_samples: float=None
    ) -> list:
        """measure_voltage 
        This method measures the voltage on a certain channel with given parameters

        Args:
            channel_name (str): SMU channel name
            range (str, optional): measurement range. Defaults to None.
            duration (float, optional): duration of measurement in seconds. Defaults to None.
            sampling_rate (float, optional): sampling rate. Defaults to None.
            number_of_samples (float, optional): number of samples to be measured. Defaults to None.

        Returns:
            list: list of measured voltages
        """
        return [self.smu_handle.channels[channel_name].measure(nidcpower.MeasurementTypes.VOLTAGE)]
        


    def measure_current(
        self, 
        channel_name: str,
        range: str=None, 
        duration: float=None, 
        sampling_rate: float=None, 
        number_of_samples: float=None
    ) -> list:
        """
        This method measures the current on a certain channel with given parameters

        Args:
            channel_name (str): SMU channel name
            range (str, optional): measurement range. Defaults to None.
            duration (float, optional): duration of measurement in seconds. Defaults to None.
            sampling_rate (float, optional): sampling rate. Defaults to None.
            number_of_samples (float, optional): number of samples to be measured. Defaults to None.

        Returns:
            list: list of measured currents
        """
        return [self.smu_handle.channels[channel_name].measure(nidcpower.MeasurementTypes.CURRENT)]


    def output_enable(self, channel_name: str, enabled: bool=False):
        """output_enable 
        This function changes the state of the device output

        Args:
            channel_name (str): channel name
            enabled (bool, optional): enable state. Defaults to False.
        """
        self.smu_handle.channels[channel_name].output_enabled = enabled


    def __del__(self):
        
        for channel in self.ni_device_dict['Details']['Channels']:
            my_logger.debug(f'Channel to be closed: {channel}')
            self.smu_handle.channels[channel].output_enabled = False
        self.smu_handle.abort()
        self.smu_handle.close()

    def GetInstrumentError(self) -> str:
        """IdentifyInstrument 
        Gets the identifier of the instrument as string

        Returns:
            ID_string [String]: The identification string from the instrument
        """
        return self.smu_handle._get_error()


    def IdentifyInstrument(self) -> str:
        """GetInstrumentError
        This function returns the error string from the instrument

        Returns:
            Error_string [String]: The error string from the instrument
        """
        return self.smu_handle.logical_name


    def ResetInstrumehnt(self) -> str:
        """ResetInstrumehnt 
        This function resets the instrument
        """
        return self.smu_handle.reset()